diff --git a/gdb/ChangeLog b/gdb/ChangeLog
index 7820302..2c8573c 100644
--- a/gdb/ChangeLog
+++ b/gdb/ChangeLog
@@ -1,3 +1,7 @@
+2016-08-02  Fredrik Hederstierna  <fredrik.hederstierna@verisure.com>
+
+	* arm-tdep.c: Fix EXC_RETURN values for ARMv6-M and ARMv7-M.
+
 2016-08-01  Joel Brobecker  <brobecker@adacore.com>
 
 	* NEWS: Create a new section for the next release branch.
diff --git a/gdb/arm-tdep.c b/gdb/arm-tdep.c
index d2661cb..43436df 100644
--- a/gdb/arm-tdep.c
+++ b/gdb/arm-tdep.c
@@ -469,9 +469,11 @@ static CORE_ADDR
 arm_addr_bits_remove (struct gdbarch *gdbarch, CORE_ADDR val)
 {
   /* On M-profile devices, do not strip the low bit from EXC_RETURN
-     (the magic exception return address).  */
+     (the magic exception return address).
+     According to B1.5.8 of both the ARMv6-M and ARMv7-M Reference Manuals
+     the EXC_RETURN value is 0xF in Bits[31:28].  */
   if (gdbarch_tdep (gdbarch)->is_m
-      && (val & 0xfffffff0) == 0xfffffff0)
+      && (val & 0xf0000000) == 0xf0000000)
     return val;
 
   if (arm_apcs_32)
@@ -2977,6 +2979,61 @@ arm_m_exception_prev_register (struct frame_info *this_frame,
 				       prev_regnum);
 }
 
+/* Determine if the program counter specified equals any of
+  these magic return values, called EXC_RETURN, defined by the
+  ARM v6-M and v7-M architectures.
+
+  From ARMv6-M Reference Manual B1.5.8
+  Table B1-5 Exception return behavior
+
+  EXC_RETURN    Return To        Return Stack
+  0xFFFFFFF1    Handler mode     Main
+  0xFFFFFFF9    Thread mode      Main
+  0xFFFFFFFD    Thread mode      Process
+
+  From ARMv7-M Reference Manual B1.5.8
+  Table B1-8 EXC_RETURN definition of exception return behavior, no FP
+
+  EXC_RETURN    Return To        Return Stack
+  0xFFFFFFF1    Handler mode     Main
+  0xFFFFFFF9    Thread mode      Main
+  0xFFFFFFFD    Thread mode      Process
+
+  Table B1-9 EXC_RETURN definition of exception return behavior, with FP
+
+  EXC_RETURN    Return To        Return Stack    Frame Type
+  0xFFFFFFE1    Handler mode     Main            Extended
+  0xFFFFFFE9    Thread mode      Main            Extended
+  0xFFFFFFED    Thread mode      Process         Extended
+  0xFFFFFFF1    Handler mode     Main            Basic
+  0xFFFFFFF9    Thread mode      Main            Basic
+  0xFFFFFFFD    Thread mode      Process         Basic
+
+  For more details see "B1.5.8 Exception return behavior"
+  in both ARMv6-M and ARMv7-M Architecture Reference Manuals.  */
+
+static int
+arm_m_pc_is_magic (CORE_ADDR pc)
+{
+  switch (pc)
+    {
+      /* Values from Tables in B1.5.8 the EXC_RETURN definitions of
+         the exception return behavior.  */
+      case 0xffffffe1:
+      case 0xffffffe9:
+      case 0xffffffed:
+      case 0xfffffff1:
+      case 0xfffffff9:
+      case 0xfffffffd:
+        /* PC is magic.  */
+        return 1;
+
+      default:
+        /* PC is not magic.  */
+        return 0;
+    }
+}
+
 /* Implementation of function hook 'sniffer' in
    'struct frame_uwnind'.  */
 
@@ -2990,14 +3047,8 @@ arm_m_exception_unwind_sniffer (const struct frame_unwind *self,
   /* No need to check is_m; this sniffer is only registered for
      M-profile architectures.  */
 
-  /* Exception frames return to one of these magic PCs.  Other values
-     are not defined as of v7-M.  See details in "B1.5.8 Exception
-     return behavior" in "ARMv7-M Architecture Reference Manual".  */
-  if (this_pc == 0xfffffff1 || this_pc == 0xfffffff9
-      || this_pc == 0xfffffffd)
-    return 1;
-
-  return 0;
+  /* Check if exception frame returns to a magic PC value.  */
+  return arm_m_pc_is_magic (this_pc);
 }
 
 /* Frame unwinder for M-profile exceptions.  */
